==> 0.0.19.tar.gz: heating-controller/src/message_processor.py <==
from loguru import logger
import json



CONVERTERS = {
    "target_temperature_output": {
      "max": lambda x: x["output_temperature"],
    },
    "window_contact_input": {
      "max": lambda x: 'closed' if (x.lower() in ('false', 'close', 'closed')) else 'open'
    }
}

# context
#   boxes: structure of boxes
#   client: MQTT client
#
# boxes['box_name']['context']
#   store here what ever is require to represent the state of the box

def prepare_context(box_name, context):
    local_context = {}
    
    local_context['id'] = box_name
    local_context['label'] = context['boxes'][box_name]['label']
    local_context['high_temperature'] = context['default_high_temperature']
    local_context['low_temperature'] = context['low_temperature']
    local_context['off_temperature'] = context['off_temperature']
    local_context['maintenance_temperature'] = context['maintenance_temperature']

    local_context['general_off'] = False
    local_context['maintenance_mode'] = False
    local_context['overwrite_window'] = False

    local_context['window_state'] = {}
    for w in context['boxes'][box_name]['windows']:
        local_context['window_state'][w['label']] = 'closed'

    local_context['mode'] = 'high'

    local_context['output_temperature'] = local_context['high_temperature']

    return local_context

def process_message(box_name, topic_key, payload, context):
    try:
        box = context['boxes'][box_name]
        local_context = box['context']
        logger.info(f"[{box_name}] Local context before: {local_context}")
        logger.info(f"[{box_name}] Processing message for '{topic_key}': {payload}")

        match topic_key.split('/'):
            case [ primary_key, sub_key ] if primary_key == 'window':
                result = process_window(box_name, context, local_context, sub_key, payload)
            case [ primary_key ] if primary_key == 'high_temp':
                result = process_high_temp(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'cmd':
                result = process_cmd(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'overwrite_window':
                result = process_overwrite_window(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'general_off':
                result = process_general_off(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'maintenance_mode':
                result = process_maintenance_mode(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'status':
                result = process_status(box_name, context, local_context, payload)
            case _:
                raise Error(f"Unexcepted topic_key: {topic_key}, {payload}") 

        if result:
            result_message = CONVERTERS["target_temperature_output"][box["output_converter"]](local_context)
            publish_topic = box["output_topic"]
            context['client'].publish(publish_topic, result_message)
            logger.info(f"[{box_name}] Result published on '{publish_topic}': {result_message}")

        context_topic = f"{context['context_topic_prefix']}{box['label']}"
        context['client'].publish(context_topic, json.dumps(local_context))

        logger.info(f"[{box_name}] Local context after: {local_context}")
    except Exception as e:
        logger.error(f"[{box_name}] Error processing '{topic_key}': {e}") 

def _calculate_output_temperature(local_context):
    # maintenance_mode has the highest priority, even higher than general_off
    if local_context['maintenance_mode']:
        local_context['output_temperature'] = local_context['maintenance_temperature']
        return
    # general_off has the next highest priority
    if local_context['general_off']:
        local_context['output_temperature'] = local_context['off_temperature']
        return
    # an open window shuts off the heating
    if not local_context['overwrite_window']:
        for w in local_context['window_state'].values():
            if w == 'open':
                local_context['output_temperature'] = local_context['off_temperature']
                return
    # finally evaluate the mode
    if local_context['mode'] == 'off':
        local_context['output_temperature'] = local_context['off_temperature']
        return
    if local_context['mode'] == 'low':
        local_context['output_temperature'] = local_context['low_temperature']
        return
    if local_context['mode'] == 'high':
        local_context['output_temperature'] = local_context['high_temperature']
        return
    # if we come here, something serious happened
    logger.error(f"Error in calculation of output_temperature: {local_context=}")
    return

def process_status(box_name, context, local_context, payload):
    return False

def process_general_off(box_name, context, local_context, payload):
    local_context['general_off'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_maintenance_mode(box_name, context, local_context, payload):
    local_context['maintenance_mode'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_cmd(box_name, context, local_context, payload):
    if payload.lower() in ('high', 'low', 'off'):
        local_context['mode'] = payload.lower()
        _calculate_output_temperature(local_context)
    else:
        logger.error(f"Invalid cmd for {box_name} received: {payload}")
    return True

def process_overwrite_window(box_name, context, local_context, payload):
    local_context['overwrite_window'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_high_temp(box_name, context, local_context, payload):
    local_context['high_temperature'] = payload
    _calculate_output_temperature(local_context)
    return True

def process_window(box_name, context, local_context, sub_key, payload):
    # default converter
    converter = lambda x:x
    for sk in context["boxes"][box_name]["windows"]:
        if sk["label"] == sub_key:
            converter = CONVERTERS["window_contact_input"][sk["converter"]]
            break
    local_context['window_state'][sub_key] = converter(payload)
    _calculate_output_temperature(local_context)
    return True

==> 0.0.21.tar.gz: heating-controller/src/message_processor.py <==
from loguru import logger
import json



CONVERTERS = {
    "target_temperature_output": {
      "max": lambda x: x["output_temperature"],
      "brennenstuhl": lambda x: json.dumps({"current_heating_setpoint":x["output_temperature"]}),
    },
    "window_contact_input": {
      "max": lambda x: 'closed' if (x.lower() in ('false', 'close', 'closed')) else 'open',
      "aqara": lambda x: 'closed' if json.loads(x)["contact"] else 'open',
    }
}

# context
#   boxes: structure of boxes
#   client: MQTT client
#
# boxes['box_name']['context']
#   store here what ever is require to represent the state of the box

def prepare_context(box_name, context):
    local_context = {}
    
    local_context['id'] = box_name
    local_context['label'] = context['boxes'][box_name]['label']
    local_context['high_temperature'] = context['default_high_temperature']
    local_context['low_temperature'] = context['low_temperature']
    local_context['off_temperature'] = context['off_temperature']
    local_context['maintenance_temperature'] = context['maintenance_temperature']

    local_context['general_off'] = False
    local_context['maintenance_mode'] = False
    local_context['overwrite_window'] = False

    local_context['window_state'] = {}
    for w in context['boxes'][box_name]['windows']:
        local_context['window_state'][w['label']] = 'closed'

    local_context['mode'] = 'high'

    local_context['output_temperature'] = local_context['high_temperature']

    return local_context

def process_message(box_name, topic_key, payload, context):
    try:
        box = context['boxes'][box_name]
        local_context = box['context']
        logger.info(f"[{box_name}] Local context before: {local_context}")
        logger.info(f"[{box_name}] Processing message for '{topic_key}': {payload}")

        match topic_key.split('/'):
            case [ primary_key, sub_key ] if primary_key == 'window':
                result = process_window(box_name, context, local_context, sub_key, payload)
            case [ primary_key ] if primary_key == 'high_temp':
                result = process_high_temp(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'cmd':
                result = process_cmd(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'overwrite_window':
                result = process_overwrite_window(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'general_off':
                result = process_general_off(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'maintenance_mode':
                result = process_maintenance_mode(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'status':
                result = process_status(box_name, context, local_context, payload)
            case _:
                raise Error(f"Unexcepted topic_key: {topic_key}, {payload}") 

        if result:
            result_message = CONVERTERS["target_temperature_output"][box["output_converter"]](local_context)
            publish_topic = box["output_topic"]
            context['client'].publish(publish_topic, result_message)
            logger.info(f"[{box_name}] Result published on '{publish_topic}': {result_message}")

        context_topic = f"{context['context_topic_prefix']}{box['label']}"
        context['client'].publish(context_topic, json.dumps(local_context))

        logger.info(f"[{box_name}] Local context after: {local_context}")
    except Exception as e:
        logger.error(f"[{box_name}] Error processing '{topic_key}': {e}") 

def _calculate_output_temperature(local_context):
    # maintenance_mode has the highest priority, even higher than general_off
    if local_context['maintenance_mode']:
        local_context['output_temperature'] = local_context['maintenance_temperature']
        return
    # general_off has the next highest priority
    if local_context['general_off']:
        local_context['output_temperature'] = local_context['off_temperature']
        return
    # an open window shuts off the heating
    if not local_context['overwrite_window']:
        for w in local_context['window_state'].values():
            if w == 'open':
                local_context['output_temperature'] = local_context['off_temperature']
                return
    # finally evaluate the mode
    if local_context['mode'] == 'off':
        local_context['output_temperature'] = local_context['off_temperature']
        return
    if local_context['mode'] == 'low':
        local_context['output_temperature'] = local_context['low_temperature']
        return
    if local_context['mode'] == 'high':
        local_context['output_temperature'] = local_context['high_temperature']
        return
    # if we come here, something serious happened
    logger.error(f"Error in calculation of output_temperature: {local_context=}")
    return

def process_status(box_name, context, local_context, payload):
    return False

def process_general_off(box_name, context, local_context, payload):
    local_context['general_off'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_maintenance_mode(box_name, context, local_context, payload):
    local_context['maintenance_mode'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_cmd(box_name, context, local_context, payload):
    if payload.lower() in ('high', 'low', 'off'):
        local_context['mode'] = payload.lower()
        _calculate_output_temperature(local_context)
    else:
        logger.error(f"Invalid cmd for {box_name} received: {payload}")
    return True

def process_overwrite_window(box_name, context, local_context, payload):
    local_context['overwrite_window'] = (payload.lower() in ('true'))
    _calculate_output_temperature(local_context)
    return True

def process_high_temp(box_name, context, local_context, payload):
    local_context['high_temperature'] = payload
    _calculate_output_temperature(local_context)
    return True

def process_window(box_name, context, local_context, sub_key, payload):
    # default converter
    converter = lambda x:x
    for sk in context["boxes"][box_name]["windows"]:
        if sk["label"] == sub_key:
            converter = CONVERTERS["window_contact_input"][sk["converter"]]
            break
    local_context['window_state'][sub_key] = converter(payload)
    _calculate_output_temperature(local_context)
    return True
